@@ -11,7 +11,7 @@ import math
 
 class hsmm:
     
-    def __init__(self, data, starts, ends, sf, cpus=1, bump_width = 50, shape=2, estimate_magnitudes=True, estimate_parameters=True,
+    def __init__(self, data, sf, cpus=1, bump_width = 50, shape=2, estimate_magnitudes=True, estimate_parameters=True,
                 parameters_to_fix = [], magnitudes_to_fix = []):
         '''
         HSMM calculates the probability of data summing over all ways of 
@@ -30,18 +30,24 @@ class hsmm:
         width : int
             width of bumps in milliseconds, originally 5 samples
         '''
+        dur_dropped_na = data.durations.dropna("trial_x_participant")
+        starts = np.roll(dur_dropped_na.data, 1)
+        starts[0] = 0
         self.starts = starts
-        self.ends = ends    
+        self.ends = dur_dropped_na.data-1 
+        self.durations =  self.ends-self.starts+1
+        self.named_durations =  data.durations.dropna("trial_x_participant") - data.durations.dropna("trial_x_participant").shift(trial_x_participant=1, fill_value=0)
         self.sf = sf
-        self.tseps = 1000/sf
-        self.n_trials = len(self.starts)  #number of trials
+        self.tseps = 1000/self.sf
+        self.n_trials = len(self.durations)  
         self.bump_width = bump_width
         self.cpus = cpus
         self.bump_width_samples = int(self.bump_width * (self.sf/1000))
         self.offset = self.bump_width_samples//2#offset on data linked to the choosen width how soon the first peak can be or how late the last,
+        self.coords = data.durations.reset_index('trial_x_participant').coords
+        data = data.data.T
         self.n_samples, self.n_dims = np.shape(data)
         self.bumps = self.calc_bumps(data)#adds bump morphology
-        self.durations = self.ends - self.starts+1#length of each trial
         self.max_d = np.max(self.durations)
         self.max_bumps = self.compute_max_bumps()
         self.shape = shape
@@ -49,7 +55,6 @@ class hsmm:
         self.estimate_parameters = estimate_parameters
         self.parameters_to_fix = parameters_to_fix
         self.magnitudes_to_fix = magnitudes_to_fix
-        
     
     def calc_bumps(self,data):
         '''
@@ -108,10 +113,9 @@ class hsmm:
             threshold for the HsMM algorithm, 0 skips HsMM
 
         '''
+        import pandas as pd 
         if verbose:
-            print(f'Estimating all solutions for {n_bumps} bumps model with {starting_points-1} random starting points')
-
-            print(f"Estimating parameters for {n_bumps} bumps model")
+            print(f'Estimating parameters for {n_bumps} bumps model with {starting_points-1} random starting points')
         if mp==True: #PCG: Dirty temporarilly needed for multiprocessing in the iterative backroll estimation...
             magnitudes = magnitudes.T
         if isinstance(parameters, (xr.DataArray,xr.Dataset)):
@@ -155,15 +159,23 @@ class hsmm:
                 parameters = np.tile([self.shape, math.ceil(np.mean(self.durations)/(n_bumps+1)/self.shape)], (n_bumps+1,1))
             if np.any(magnitudes)== None:
                 magnitudes = np.zeros((self.n_dims,n_bumps))
+            initial_p = parameters
+            initial_m = magnitudes
             lkh, mags, pars, eventprobs = self.fit(n_bumps, magnitudes, parameters, threshold)
         else:
+            initial_p = parameters
+            initial_m = magnitudes
             lkh, mags, pars, eventprobs = self.fit(n_bumps, magnitudes, parameters, threshold)
         #Comparing to uninitialized gamma parameters
-        #parameters = np.tile([self.shape, 50], (n_bumps+1,1))
-        #likelihood, magnitudes_, parameters_, eventprobs_ = \
-        #        self.fit(n_bumps, magnitudes, parameters, threshold)
-        #if likelihood > lkh:
-        #    lkh, mags, pars, eventprobs = likelihood, magnitudes_, parameters_, eventprobs_
+        if starting_points == 1:
+            parameters = np.tile([self.shape, 50], (n_bumps+1,1))
+            magnitudes = np.zeros((self.n_dims,n_bumps))
+            likelihood, magnitudes_, parameters_, eventprobs_ = \
+                    self.fit(n_bumps, initial_m, parameters, threshold)
+            if likelihood > lkh:
+                if verbose:
+                    print('Likelihood of uninitialized parameters has been preferred over initialized model. Consider adding starting points?')
+                lkh, mags, pars, eventprobs = likelihood, magnitudes_, parameters_, eventprobs_
 
         
         if len(pars) != self.max_bumps+1:#align all dimensions
@@ -175,8 +187,21 @@ class hsmm:
         xrlikelihoods = xr.DataArray(lkh , name="likelihoods")
         xrparams = xr.DataArray(pars, dims=("stage",'params'), name="parameters")
         xrmags = xr.DataArray(mags, dims=("component","bump"), name="magnitudes")
-        xreventprobs = xr.DataArray(eventprobs, dims=("samples",'trial','bump'), name="eventprobs")
-        estimated = xr.merge((xrlikelihoods, xrparams, xrmags, xreventprobs))#,xreventprobs))
+        #xreventprobs = xr.DataArray(eventprobs, dims=("samples",'trialxPart','bump'), name="eventprobs")
+        #xreventprobs = xreventprobs.trialxPart.set_index(self.coords.values)
+        part, trial = self.coords['subjects'].values, self.coords['trials'].values
+
+        n_samples, n_participant_x_trials, bumps_n = np.shape(eventprobs)
+        xreventprobs = xr.Dataset({'eventprobs': (('bump', 'trial_x_participant','samples'), 
+                                         eventprobs.T)},
+                         {'bump':np.arange(bumps_n),
+                          'samples':np.arange(n_samples),
+                        'trial_x_participant':  pd.MultiIndex.from_arrays([part,trial],
+                                names=('participant','trials'))})
+        #xreventprobs = xreventprobs.unstack('trial_x_participant')
+        xreventprobs = xreventprobs.transpose('trial_x_participant','samples','bump')
+        estimated = xr.merge((xrlikelihoods, xrparams, xrmags, xreventprobs))
+
         if verbose:
             print(f"Parameters estimated for {n_bumps} bumps model")
         return estimated
@@ -393,6 +418,7 @@ class hsmm:
             bump_loo_results = [max_fit]
         i = 0
         for n_bumps in np.arange(self.max_bumps-1,0,-1):
+            print(f'Estimating all solutions for {n_bumps} number of bumps')
             temp_best = bump_loo_results[i]#previous bump solution
             temp_best = temp_best.dropna('bump')
             temp_best = temp_best.dropna('stage')
@@ -409,9 +435,7 @@ class hsmm:
             if self.cpus > 1:
                 with mp.Pool(processes=self.cpus) as pool:
                     bump_loo_likelihood_temp = pool.starmap(self.fit_single, 
-                        zip(itertools.repeat(n_bumps), bumps_temp, flats_temp,#itertools.repeat(np.tile([self.shape,50], (n_bumps+1,1))),# ##
-                            #temp_best.parameters.values[possible_flats,:],
-                            #itertools.repeat(self.get_init_parameters(n_bumps)),
+                        zip(itertools.repeat(n_bumps), bumps_temp, flats_temp,
                             itertools.repeat(1),itertools.repeat(True),itertools.repeat(False)))
             else:
                 raise ValueError('For loop not yet written use cpus >1')
@@ -455,7 +479,7 @@ class hsmm:
         '''
         return int(np.min(self.durations)/self.bump_width_samples)
 
-    def bump_times(self, eventprobs):
+    def bump_times(self, eventprobs, mean=True):
         '''
         Compute bump onset times based on bump probabilities
 
@@ -474,12 +498,44 @@ class hsmm:
             density for a gamma with given parameters
         '''    
         
-        eventprobs = eventprobs.dropna('bump')
-        onsets = np.empty((len(eventprobs.trial),len(eventprobs.bump)+1))
+        eventprobs = eventprobs.dropna('bump', how="all")
+        eventprobs = eventprobs.dropna('trial_x_participant', how="all")
+        onsets = np.empty((len(eventprobs.trial_x_participant),len(eventprobs.bump)+1))
         i = 0
-        for trial in eventprobs.trial.values:
-            onsets[i, :len(eventprobs.bump)] = np.arange(self.max_d) @ eventprobs.sel(trial=trial).data - self.bump_width_samples/2#Correcting for centerning, thus times represents bump onset
+        for trial in eventprobs.trial_x_participant.dropna('trial_x_participant', how="all").values:
+            onsets[i, :len(eventprobs.bump)] = np.arange(self.max_d) @ eventprobs.sel(trial_x_participant=trial).data - self.bump_width_samples/2#Correcting for centerning, thus times represents bump onset
             onsets[i, -1] = self.ends[i] - self.starts[i]
             i += 1
-        return onsets
-    
\ No newline at end of file
+        if mean:
+            return np.mean(onsets, axis=0)
+        else:
+            return onsets
+    
+    def compute_topo(self, data, eventprobs, mean=True):
+        if 'trial_x_participant' not in data:
+            data = data.stack(trial_x_participant=['participant','epochs'])
+        eventprobs = eventprobs.dropna('bump', how="all")
+        eventprobs = eventprobs.dropna('trial_x_participant', how="all")
+        topologies = np.empty((len(eventprobs.trial_x_participant.data), 
+                               len(eventprobs.bump.data),
+                               len(data.electrodes.data)))
+        i = 0
+        for trial_x_participant in eventprobs.trial_x_participant:
+            z = 0
+            for bump in eventprobs.bump:
+                trial_samples = np.arange(self.named_durations.sel(trial_x_participant=trial_x_participant).data)
+                topologies[i,z, :] = data.sel(trial_x_participant = trial_x_participant,
+                                              samples=trial_samples).data @ \
+                    eventprobs.sel(trial_x_participant = trial_x_participant,  bump=bump, samples=trial_samples)
+                z += 1
+            i += 1
+        if mean:
+            return np.mean(topologies, axis=0)
+        else:
+            return topologies
+        
+    def multiple_topologies(self, data, eventprobs, mean=True):
+        topo = np.tile(np.nan, (len(eventprobs.n_bumps), len(eventprobs.n_bumps), len(data.electrodes)))
+        for n_bumps in eventprobs.n_bumps:
+            topo[n_bumps-1, :n_bumps.values, :] = self.compute_topo(data, eventprobs.sel(n_bumps=n_bumps))
+        return topo
\ No newline at end of file
